@@ -1,3 +1,22 @@
+/*
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
+ */
+
 #include <glib/gprintf.h>
 #include <string.h>
 
@@ -122,7 +141,7 @@ print_variablelist_entry_list (FILE        *f,
         g_fprintf (f, "</varlistentry>\n");
 }
 
-
+#if 0
 static void
 print_deprecated_message (FILE *f)
 {
@@ -131,6 +150,7 @@ print_deprecated_message (FILE *f)
         g_fprintf (f, "<para>This item is deprecated</para>\n");
         g_fprintf (f, "</note>\n");
 }
+#endif
 
 static void
 print_sgml_header (FILE *f, OntologyDescription *desc)
@@ -142,7 +162,7 @@ print_sgml_header (FILE *f, OntologyDescription *desc)
         g_fprintf (f, "<chapter id='%s-ontology'>\n", desc->localPrefix);
 
         upper_name = g_ascii_strup (desc->localPrefix, -1);
-        g_fprintf (f, "<title>%s Ontology</title>\n", upper_name);
+        g_fprintf (f, "<title>%s: %s</title>\n", desc->title, desc->description ? desc->description : "");
         g_free (upper_name);
 
         print_people_list (f, "Authors:", desc->authors);
