@@ -56,6 +56,7 @@ typedef struct {
 	DBusPendingCall *call;
 	TrackerDBusSendAndSpliceCallback callback;
 	gpointer user_data;
+	gboolean expect_variable_names;
 } SendAndSpliceData;
 
 static GSList *hooks;
@@ -705,6 +706,31 @@ tracker_dbus_enable_client_lookup (gboolean enabled)
 	client_lookup_enabled = enabled;
 }
 
+static GStrv
+dbus_send_and_splice_get_variable_names (DBusMessage *message,
+                                         gboolean     copy_strings)
+{
+	GPtrArray *found;
+	DBusMessageIter iter, arr;
+
+	dbus_message_iter_init (message, &iter);
+	dbus_message_iter_recurse (&iter, &arr);
+
+	found = g_ptr_array_new ();
+
+	while (dbus_message_iter_get_arg_type (&arr) != DBUS_TYPE_INVALID) {
+		gchar *str;
+
+		dbus_message_iter_get_basic (&arr, &str);
+		g_ptr_array_add (found, copy_strings ? g_strdup (str) : str);
+		dbus_message_iter_next (&arr);
+	}
+
+	g_ptr_array_add (found, NULL);
+
+	return g_ptr_array_free (found, FALSE);
+}
+
 /*
  * /!\ BIG FAT WARNING /!\
  * The message must be destroyed for this function to succeed, so pass a
@@ -718,6 +744,7 @@ tracker_dbus_send_and_splice (DBusConnection  *connection,
                               GCancellable    *cancellable,
                               void           **dest_buffer,
                               gssize          *dest_buffer_size,
+                              GStrv           *variable_names,
                               GError         **error)
 {
 	DBusPendingCall *call;
@@ -773,6 +800,10 @@ tracker_dbus_send_and_splice (DBusConnection  *connection,
 			dbus_set_error_from_message (&dbus_error, reply);
 			dbus_set_g_error (error, &dbus_error);
 			dbus_error_free (&dbus_error);
+
+			/* If any error happened, we're not passing any received data, so we
+			 * need to free it */
+			g_free (g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (output_stream)));
 		} else {
 			*dest_buffer = g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (output_stream));
 
@@ -780,6 +811,10 @@ tracker_dbus_send_and_splice (DBusConnection  *connection,
 				*dest_buffer_size = g_memory_output_stream_get_data_size (G_MEMORY_OUTPUT_STREAM (output_stream));
 			}
 
+			if (variable_names) {
+				*variable_names = dbus_send_and_splice_get_variable_names (reply, TRUE);
+			}
+
 			ret_value = TRUE;
 		}
 	} else {
@@ -788,6 +823,9 @@ tracker_dbus_send_and_splice (DBusConnection  *connection,
 		             TRACKER_DBUS_ERROR_BROKEN_PIPE,
 		             "Couldn't get results from server");
 		g_error_free (inner_error);
+		/* If any error happened, we're not passing any received data, so we
+		 * need to free it */
+		g_free (g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (output_stream)));
 	}
 
 	g_object_unref (output_stream);
@@ -807,6 +845,7 @@ static SendAndSpliceData *
 send_and_splice_data_new (GInputStream                     *unix_input_stream,
                           GInputStream                     *buffered_input_stream,
                           GOutputStream                    *output_stream,
+                          gboolean                          expect_variable_names,
                           DBusPendingCall                  *call,
                           TrackerDBusSendAndSpliceCallback  callback,
                           gpointer                          user_data)
@@ -820,6 +859,7 @@ send_and_splice_data_new (GInputStream                     *unix_input_stream,
 	data->call = call;
 	data->callback = callback;
 	data->user_data = user_data;
+	data->expect_variable_names = expect_variable_names;
 
 	return data;
 }
@@ -854,25 +894,50 @@ send_and_splice_async_callback (GObject      *source,
 		if (G_UNLIKELY (dbus_message_get_type (reply) == DBUS_MESSAGE_TYPE_ERROR)) {
 			DBusError dbus_error;
 
+			/* If any error happened, we're not passing any received data, so we
+			 * need to free it */
+			g_free (g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (data->output_stream)));
+
 			dbus_error_init (&dbus_error);
 			dbus_set_error_from_message (&dbus_error, reply);
 			dbus_set_g_error (&error, &dbus_error);
 			dbus_error_free (&dbus_error);
 
-			(* data->callback) (NULL, -1, error, data->user_data);
+			(* data->callback) (NULL, -1, NULL, error, data->user_data);
 
-			g_error_free (error);
+			/* Note: GError should be freed by callback. We do this to be aligned
+			 * with the behavior of dbus-glib, where if an error happens, the
+			 * GError passed to the callback is supposed to be disposed by the
+			 * callback itself. */
 		} else {
+			GStrv v_names = NULL;
+
+			if (data->expect_variable_names) {
+				v_names = dbus_send_and_splice_get_variable_names (reply, FALSE);
+			}
+
 			dbus_pending_call_cancel (data->call);
+
 			(* data->callback) (g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (data->output_stream)),
 			                    g_memory_output_stream_get_data_size (G_MEMORY_OUTPUT_STREAM (data->output_stream)),
+			                    v_names,
 			                    NULL,
 			                    data->user_data);
+
+			/* Don't use g_strfreev here, see above */
+			g_free (v_names);
 		}
 	} else {
-		(* data->callback) (NULL, -1, error, data->user_data);
+		/* If any error happened, we're not passing any received data, so we
+		 * need to free it */
+		g_free (g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (data->output_stream)));
 
-		g_error_free (error);
+		(* data->callback) (NULL, -1, NULL, error, data->user_data);
+
+		/* Note: GError should be freed by callback. We do this to be aligned
+		 * with the behavior of dbus-glib, where if an error happens, the
+		 * GError passed to the callback is supposed to be disposed by the
+		 * callback itself. */
 	}
 
 	if (reply) {
@@ -886,6 +951,7 @@ gboolean
 tracker_dbus_send_and_splice_async (DBusConnection                   *connection,
                                     DBusMessage                      *message,
                                     int                               fd,
+                                    gboolean                          expect_variable_names,
                                     GCancellable                     *cancellable,
                                     TrackerDBusSendAndSpliceCallback  callback,
                                     gpointer                          user_data)
@@ -920,6 +986,7 @@ tracker_dbus_send_and_splice_async (DBusConnection                   *connection
 	data = send_and_splice_data_new (unix_input_stream,
 	                                 buffered_input_stream,
 	                                 output_stream,
+	                                 expect_variable_names,
 	                                 call,
 	                                 callback,
 	                                 user_data);
