@@ -25,7 +25,7 @@
 
 #include <glib.h>
 
-#include <libtracker-db/tracker-db-interface.h>
+#include "tracker-db-interface.h"
 
 G_BEGIN_DECLS
 
