@@ -34,9 +34,12 @@ G_BEGIN_DECLS
 #error "only <libtracker-data/tracker-data.h> must be included directly."
 #endif
 
-typedef void (*TrackerStatementCallback) (const gchar *graph,
+typedef void (*TrackerStatementCallback) (gint         graph_id,
+                                          const gchar *graph,
+                                          gint         subject_id,
                                           const gchar *subject,
-                                          const gchar *predicate,
+                                          gint         predicate_id,
+                                          gint         object_id,
                                           const gchar *object,
                                           GPtrArray   *rdf_types,
                                           gpointer     user_data);
