@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, Nokia (urho.konttori@nokia.com)
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,6 +27,7 @@
 #include "tracker-miner-manager.h"
 #include "tracker-marshal.h"
 #include "tracker-miner-client.h"
+#include "tracker-miner-files-index-client.h"
 #include "tracker-miner-dbus.h"
 
 /**
@@ -219,9 +220,6 @@ name_owner_changed_cb (DBusGProxy  *proxy,
                        gpointer     user_data)
 {
 	TrackerMinerManager *manager = user_data;
-	TrackerMinerManagerPrivate *priv;
-
-	priv = TRACKER_MINER_MANAGER_GET_PRIVATE (manager);
 
 	if (find_miner_proxy (manager, name, FALSE) != NULL) {
 		if (new_owner && (!old_owner || !*old_owner)) {
@@ -983,3 +981,108 @@ tracker_miner_manager_ignore_next_update (TrackerMinerManager *manager,
 
 	return TRUE;
 }
+
+GQuark
+tracker_miner_manager_error_quark (void)
+{
+	static GQuark error_quark = 0;
+
+	if (G_UNLIKELY (error_quark == 0)) {
+		error_quark = g_quark_from_static_string ("tracker-miner-manager-error-quark");
+	}
+
+	return error_quark;
+}
+
+gboolean
+tracker_miner_manager_reindex_by_mimetype (TrackerMinerManager  *manager,
+                                           const GStrv           mimetypes,
+                                           GError              **error)
+{
+	static DBusGProxy *proxy = NULL;
+	TrackerMinerManagerPrivate *priv;
+	GError *internal_error = NULL;
+
+	g_return_val_if_fail (TRACKER_IS_MINER_MANAGER (manager), FALSE);
+	g_return_val_if_fail (mimetypes != NULL, FALSE);
+
+	if (!tracker_miner_manager_is_active (manager,
+	                                      "org.freedesktop.Tracker1.Miner.Files")) {
+		g_set_error_literal (error,
+		                     TRACKER_MINER_MANAGER_ERROR,
+		                     TRACKER_MINER_MANAGER_ERROR_NOT_AVAILABLE,
+		                     "Filesystem miner is not active");
+		return FALSE;
+	}
+
+	priv = TRACKER_MINER_MANAGER_GET_PRIVATE (manager);
+
+	if (G_UNLIKELY (!proxy)) {
+		proxy = dbus_g_proxy_new_for_name (priv->connection,
+		                                   "org.freedesktop.Tracker1.Miner.Files.Index",
+		                                   "/org/freedesktop/Tracker1/Miner/Files/Index",
+		                                   "org.freedesktop.Tracker1.Miner.Files.Index");
+	}
+
+	org_freedesktop_Tracker1_Miner_Files_Index_reindex_mime_types (proxy,
+	                                                               (const gchar **) mimetypes,
+	                                                               &internal_error);
+
+	if (internal_error) {
+		g_propagate_error (error, internal_error);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+gboolean
+tracker_miner_manager_index_file (TrackerMinerManager  *manager,
+                                  GFile                *file,
+                                  GError              **error)
+{
+	static DBusGProxy *proxy = NULL;
+	TrackerMinerManagerPrivate *priv;
+	GError *internal_error = NULL;
+	gchar *uri;
+
+	g_return_val_if_fail (TRACKER_IS_MINER_MANAGER (manager), FALSE);
+	g_return_val_if_fail (G_IS_FILE (file), FALSE);
+
+	if (!g_file_query_exists (file, NULL)) {
+		g_set_error_literal (error,
+		                     TRACKER_MINER_MANAGER_ERROR,
+		                     TRACKER_MINER_MANAGER_ERROR_NOENT,
+		                     "File or directory does not exist");
+		return FALSE;
+	}
+
+	if (!tracker_miner_manager_is_active (manager,
+	                                      "org.freedesktop.Tracker1.Miner.Files")) {
+		g_set_error_literal (error,
+		                     TRACKER_MINER_MANAGER_ERROR,
+		                     TRACKER_MINER_MANAGER_ERROR_NOT_AVAILABLE,
+		                     "Filesystem miner is not active");
+		return FALSE;
+	}
+
+	priv = TRACKER_MINER_MANAGER_GET_PRIVATE (manager);
+
+	if (G_UNLIKELY (!proxy)) {
+		proxy = dbus_g_proxy_new_for_name (priv->connection,
+		                                   "org.freedesktop.Tracker1.Miner.Files.Index",
+		                                   "/org/freedesktop/Tracker1/Miner/Files/Index",
+		                                   "org.freedesktop.Tracker1.Miner.Files.Index");
+	}
+
+	uri = g_file_get_uri (file);
+	org_freedesktop_Tracker1_Miner_Files_Index_index_file (proxy, uri, &internal_error);
+	g_free (uri);
+
+	if (internal_error) {
+		g_propagate_error (error, internal_error);
+		return FALSE;
+	}
+
+	return TRUE;
+}
