@@ -29,8 +29,8 @@
 
 #include <jpeglib.h>
 
-#include <libtracker-common/tracker-file-utils.h>
-#include <libtracker-client/tracker-client.h>
+#include <libtracker-common/tracker-common.h>
+
 #include <libtracker-extract/tracker-extract.h>
 
 #include "tracker-main.h"
@@ -54,7 +54,8 @@
 #endif /* HAVE_LIBIPTCDATA */
 
 typedef struct {
-	gchar *camera;
+	const gchar *make;
+	const gchar *model;
 	const gchar *title;
 	const gchar *orientation;
 	const gchar *copyright;
@@ -233,23 +234,6 @@ extract_jpeg (const gchar          *uri,
 		id = g_new0 (TrackerIptcData, 1);
 	}
 
-	/* Don't merge if the make is in the model */
-	if ((xd->make == NULL || xd->model == NULL) ||
-	    (xd->make && xd->model && strstr (xd->model, xd->make) == NULL)) {
-		md.camera = tracker_merge_const (" ", 2, xd->make, xd->model);
-	} else {
-		md.camera = g_strdup (xd->model);
-	}
-
-	if (!md.camera) {
-		if ((ed->make == NULL || ed->model == NULL) ||
-		    (ed->make && ed->model && strstr (ed->model, ed->make) == NULL)) {
-			md.camera = tracker_merge_const (" ", 2, ed->make, ed->model);
-		} else {
-			md.camera = g_strdup (ed->model);
-		}
-	}
-
 	md.title = tracker_coalesce_strip (4, xd->title, ed->document_name, xd->title2, xd->pdf_title);
 	md.orientation = tracker_coalesce_strip (3, xd->orientation, ed->orientation, id->image_orientation);
 	md.copyright = tracker_coalesce_strip (4, xd->copyright, xd->rights, ed->copyright, id->copyright_notice);
@@ -269,6 +253,8 @@ extract_jpeg (const gchar          *uri,
 	md.country = tracker_coalesce_strip (2, xd->country, id->country_name);
 	md.creator = tracker_coalesce_strip (3, xd->creator, id->byline, id->credit);
 	md.comment = tracker_coalesce_strip (2, comment, ed->user_comment);
+	md.make = tracker_coalesce_strip (2, xd->make, ed->make);
+	md.model = tracker_coalesce_strip (2, xd->model, ed->model);
 
 	/* Prioritize on native dimention in all cases */
 	tracker_sparql_builder_predicate (metadata, "nfo:width");
@@ -280,7 +266,7 @@ extract_jpeg (const gchar          *uri,
 	tracker_sparql_builder_object_int64 (metadata, cinfo.image_height);
 
 	if (id->contact) {
-		gchar *uri = tracker_uri_printf_escaped ("urn:contact:%s", id->contact);
+		gchar *uri = tracker_sparql_escape_uri_printf ("urn:contact:%s", id->contact);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
 		tracker_sparql_builder_subject_iri (preupdate, uri);
@@ -310,7 +296,7 @@ extract_jpeg (const gchar          *uri,
 	}
 
 	if (xd->publisher) {
-		gchar *uri = tracker_uri_printf_escaped ("urn:contact:%s", xd->publisher);
+		gchar *uri = tracker_sparql_escape_uri_printf ("urn:contact:%s", xd->publisher);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
 		tracker_sparql_builder_subject_iri (preupdate, uri);
@@ -391,9 +377,30 @@ extract_jpeg (const gchar          *uri,
 	}
 	g_ptr_array_free (keywords, TRUE);
 
-	if (md.camera) {
-		tracker_sparql_builder_predicate (metadata, "nfo:device");
-		tracker_sparql_builder_object_unvalidated (metadata, md.camera);
+	if (md.make || md.model) {
+		gchar *equip_uri;
+
+		equip_uri = tracker_sparql_escape_uri_printf ("urn:equipment:%s:%s:",
+		                                              md.make ? md.make : "",
+		                                              md.model ? md.model : "");
+
+		tracker_sparql_builder_insert_open (preupdate, NULL);
+		tracker_sparql_builder_subject_iri (preupdate, equip_uri);
+		tracker_sparql_builder_predicate (preupdate, "a");
+		tracker_sparql_builder_object (preupdate, "nfo:Equipment");
+
+		if (md.make) {
+			tracker_sparql_builder_predicate (preupdate, "nfo:manufacturer");
+			tracker_sparql_builder_object_unvalidated (preupdate, md.make);
+		}
+		if (md.model) {
+			tracker_sparql_builder_predicate (preupdate, "nfo:model");
+			tracker_sparql_builder_object_unvalidated (preupdate, md.model);
+		}
+		tracker_sparql_builder_insert_close (preupdate);
+		tracker_sparql_builder_predicate (metadata, "nfo:equipment");
+		tracker_sparql_builder_object_iri (metadata, equip_uri);
+		g_free (equip_uri);
 	}
 
 	if (md.title) {
@@ -438,7 +445,7 @@ extract_jpeg (const gchar          *uri,
 	}
 
 	if (md.artist) {
-		gchar *uri = tracker_uri_printf_escaped ("urn:contact:%s", md.artist);
+		gchar *uri = tracker_sparql_escape_uri_printf ("urn:contact:%s", md.artist);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
 		tracker_sparql_builder_subject_iri (preupdate, uri);
@@ -485,7 +492,7 @@ extract_jpeg (const gchar          *uri,
 	}
 
 	if (md.creator) {
-		gchar *uri = tracker_uri_printf_escaped ("urn:contact:%s", md.creator);
+		gchar *uri = tracker_sparql_escape_uri_printf ("urn:contact:%s", md.creator);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
 		tracker_sparql_builder_subject_iri (preupdate, uri);
@@ -598,8 +605,6 @@ extract_jpeg (const gchar          *uri,
 
 	jpeg_destroy_decompress (&cinfo);
 
-	g_free (md.camera);
-
 	tracker_exif_free (ed);
 	tracker_xmp_free (xd);
 	tracker_iptc_free (id);
