@@ -27,7 +27,8 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 
-#include <libtracker-client/tracker.h>
+#include <libtracker-sparql/tracker-sparql.h>
+
 #include <libtracker-common/tracker-common.h>
 
 #define ABOUT	  \
@@ -56,6 +57,7 @@ static gboolean video_files;
 static gboolean document_files;
 static gboolean emails;
 static gboolean contacts;
+static gboolean feeds;
 static gboolean print_version;
 
 static GOptionEntry entries[] = {
@@ -72,7 +74,7 @@ static GOptionEntry entries[] = {
 	  NULL
 	},
 	{ "detailed", 'd', 0, G_OPTION_ARG_NONE, &detailed,
-	  N_("Show URNs for results (doesn't apply to --music-albums, --music-artists)"),
+	  N_("Show URNs for results (doesn't apply to --music-albums, --music-artists, --feeds)"),
 	  NULL
 	},
 	{ "all", 'a', 0, G_OPTION_ARG_NONE, &all,
@@ -119,6 +121,10 @@ static GOptionEntry entries[] = {
 	  N_("Search for contacts"),
 	  NULL
 	},
+	{ "feeds", 0, 0, G_OPTION_ARG_NONE, &feeds,
+	  N_("Search for feeds (--all has no effect on this) "),
+	  NULL
+	},
 	{ "version", 'V', 0, G_OPTION_ARG_NONE, &print_version,
 	  N_("Print version"),
 	  NULL
@@ -150,9 +156,11 @@ static gchar *
 get_fts_string (GStrv    search_words,
                 gboolean use_or_operator)
 {
+#if HAVE_TRACKER_FTS
 	GString *fts;
 	gint i, len;
 
+
 	if (!search_words) {
 		return NULL;
 	}
@@ -161,8 +169,13 @@ get_fts_string (GStrv    search_words,
 	len = g_strv_length (search_words);
 
 	for (i = 0; i < len; i++) {
-		g_string_append (fts, search_words[i]);
-		g_string_append_c (fts, '*');
+		gchar *escaped;
+
+		/* Properly escape the input string as it's going to be passed
+		 * in a sparql query */
+		escaped = tracker_sparql_escape_string (search_words[i]);
+
+		g_string_append (fts, escaped);
 
 		if (i < len - 1) {
 			if (use_or_operator) {
@@ -171,38 +184,27 @@ get_fts_string (GStrv    search_words,
 				g_string_append (fts, " ");
 			}
 		}
+
+		g_free (escaped);
 	}
 
 	return g_string_free (fts, FALSE);
-}
-
-static void
-get_contacts_foreach (gpointer value,
-		    gpointer user_data)
-{
-	gchar **data;
-	gboolean details;
-
-	data = value;
-	details = GPOINTER_TO_INT (user_data);
-
-	if (details && data[1] && *data[1]) {
-		g_print ("  %s, %s (%s)\n", data[0], data[1], data[2]);
-	} else {
-		g_print ("  %s, %s\n", data[0], data[1]);
-	}
+#else
+	/* If FTS support not enabled, always do non-fts searches */
+	return NULL;
+#endif
 }
 
 static gboolean
-get_contacts_results (TrackerClient *client,
-		      const gchar   *query,
-		      gint           search_limit,
-		      gboolean       details)
+get_contacts_results (TrackerSparqlConnection *connection,
+                      const gchar             *query,
+                      gint                     search_limit,
+                      gboolean                 details)
 {
 	GError *error = NULL;
-	GPtrArray *results;
+	TrackerSparqlCursor *cursor;
 
-	results = tracker_resources_sparql_query (client, query, &error);
+	cursor = tracker_sparql_connection_query (connection, query, NULL, &error);
 
 	if (error) {
 		g_printerr ("%s, %s\n",
@@ -213,40 +215,49 @@ get_contacts_results (TrackerClient *client,
 		return FALSE;
 	}
 
-	if (!results) {
+	if (!cursor) {
 		g_print ("%s\n",
 		         _("No contacts were found"));
 	} else {
-		g_print (g_dngettext (NULL,
-		                      "Contact: %d",
-		                      "Contacts: %d",
-		                      results->len),
-		         results->len);
+		gint count = 0;
+
+		g_print ("%s:\n", _("Contacts"));
+
+		while (tracker_sparql_cursor_next (cursor, NULL, NULL)) {
+			if (details) {
+				g_print ("  '%s', %s (%s)\n",
+				         tracker_sparql_cursor_get_string (cursor, 0, NULL),
+				         tracker_sparql_cursor_get_string (cursor, 1, NULL),
+				         tracker_sparql_cursor_get_string (cursor, 2, NULL));
+			} else {
+				g_print ("  '%s', %s\n",
+				         tracker_sparql_cursor_get_string (cursor, 0, NULL),
+				         tracker_sparql_cursor_get_string (cursor, 1, NULL));
+			}
+
+			count++;
+		}
+
 		g_print ("\n");
 
-		g_ptr_array_foreach (results,
-		                     get_contacts_foreach,
-		                     GINT_TO_POINTER (details));
-
-		if (results->len >= search_limit) {
+		if (count >= search_limit) {
 			show_limit_warning ();
 		}
 
-		g_ptr_array_foreach (results, (GFunc) g_strfreev, NULL);
-		g_ptr_array_free (results, TRUE);
+		g_object_unref (cursor);
 	}
 
 	return TRUE;
 }
 
 static gboolean
-get_contacts (TrackerClient *client,
-	      GStrv          search_terms,
-	      gboolean       show_all,
-	      gint           search_offset,
-	      gint           search_limit,
-	      gboolean       use_or_operator,
-	      gboolean       details)
+get_contacts (TrackerSparqlConnection *connection,
+              GStrv                    search_terms,
+              gboolean                 show_all,
+              gint                     search_offset,
+              gint                     search_limit,
+              gboolean                 use_or_operator,
+              gboolean                 details)
 {
 	gchar *fts;
 	gchar *query;
@@ -255,7 +266,7 @@ get_contacts (TrackerClient *client,
 	fts = get_fts_string (search_terms, use_or_operator);
 
 	if (fts) {
-		query = g_strdup_printf ("SELECT tracker:coalesce(nco:fullname(?contact), \"Unknown\") nco:hasEmailAddress(?contact) ?contact "
+		query = g_strdup_printf ("SELECT tracker:coalesce(nco:fullname(?contact), \"%s\") tracker:coalesce(nco:hasEmailAddress(?contact), \"%s\") ?contact "
 		                         "WHERE { "
 		                         "  ?contact a nco:Contact ;"
 		                         "  fts:match \"%s\" ."
@@ -263,55 +274,42 @@ get_contacts (TrackerClient *client,
 		                         "ORDER BY ASC(nco:fullname(?contact)) ASC(nco:hasEmailAddress(?contact)) "
 		                         "OFFSET %d "
 		                         "LIMIT %d",
+                                         _("No Name"),
+                                         _("No Address"),
 		                         fts,
 		                         search_offset,
 		                         search_limit);
 	} else {
-		query = g_strdup_printf ("SELECT tracker:coalesce(nco:fullname(?contact), \"Unknown\") nco:hasEmailAddress(?contact) ?contact "
+		query = g_strdup_printf ("SELECT tracker:coalesce(nco:fullname(?contact), \"%s\") tracker:coalesce(nco:hasEmailAddress(?contact), \"%s\") ?contact "
 		                         "WHERE { "
 		                         "  ?contact a nco:Contact ."
 		                         "} "
 		                         "ORDER BY ASC(nco:fullname(?contact)) ASC(nco:hasEmailAddress(?contact)) "
 		                         "OFFSET %d "
 		                         "LIMIT %d",
+                                         _("No Name"),
+                                         _("No Address"),
 		                         search_offset,
 		                         search_limit);
 	}
 
-	success = get_contacts_results (client, query, search_limit, details);
+	success = get_contacts_results (connection, query, search_limit, details);
 	g_free (query);
 	g_free (fts);
 
 	return success;
 }
 
-static void
-get_emails_foreach (gpointer value,
-		    gpointer user_data)
-{
-	gchar **data;
-	gboolean details;
-
-	data = value;
-	details = GPOINTER_TO_INT (user_data);
-
-	if (details && data[1] && *data[1]) {
-		g_print ("  %s, %s (%s)\n", data[0], data[1], data[2]);
-	} else {
-		g_print ("  %s, %s\n", data[0], data[1]);
-	}
-}
-
 static gboolean
-get_emails_results (TrackerClient *client,
-		    const gchar   *query,
-		    gint           search_limit,
-		    gboolean       details)
+get_emails_results (TrackerSparqlConnection *connection,
+                    const gchar             *query,
+                    gint                     search_limit,
+                    gboolean                 details)
 {
 	GError *error = NULL;
-	GPtrArray *results;
+	TrackerSparqlCursor *cursor;
 
-	results = tracker_resources_sparql_query (client, query, &error);
+	cursor = tracker_sparql_connection_query (connection, query, NULL, &error);
 
 	if (error) {
 		g_printerr ("%s, %s\n",
@@ -322,47 +320,54 @@ get_emails_results (TrackerClient *client,
 		return FALSE;
 	}
 
-	if (!results) {
+	if (!cursor) {
 		g_print ("%s\n",
 		         _("No emails were found"));
 	} else {
-		g_print (g_dngettext (NULL,
-		                      "Email: %d",
-		                      "Emails: %d",
-		                      results->len),
-		         results->len);
+		gint count = 0;
+
+		g_print ("%s:\n", _("Emails"));
+
+		while (tracker_sparql_cursor_next (cursor, NULL, NULL)) {
+			if (details) {
+				g_print ("  %s, %s (%s)\n",
+				         tracker_sparql_cursor_get_string (cursor, 0, NULL),
+				         tracker_sparql_cursor_get_string (cursor, 1, NULL),
+				         tracker_sparql_cursor_get_string (cursor, 2, NULL));
+			} else {
+				g_print ("  %s, %s\n",
+				         tracker_sparql_cursor_get_string (cursor, 0, NULL),
+				         tracker_sparql_cursor_get_string (cursor, 1, NULL));
+			}
+
+			count++;
+		}
+
 		g_print ("\n");
 
-		g_ptr_array_foreach (results,
-		                     get_emails_foreach,
-		                     GINT_TO_POINTER (details));
-
-		if (results->len >= search_limit) {
+		if (count >= search_limit) {
 			show_limit_warning ();
 		}
 
-		g_ptr_array_foreach (results, (GFunc) g_strfreev, NULL);
-		g_ptr_array_free (results, TRUE);
+		g_object_unref (cursor);
 	}
 
 	return TRUE;
 }
 
 static gboolean
-get_emails (TrackerClient *client,
-	    GStrv          search_terms,
-	    gboolean       show_all,
-	    gint           search_offset,
-	    gint           search_limit,
-	    gboolean       use_or_operator,
-	    gboolean       details)
+get_emails (TrackerSparqlConnection *connection,
+            GStrv                    search_terms,
+            gboolean                 show_all,
+            gint                     search_offset,
+            gint                     search_limit,
+            gboolean                 use_or_operator,
+            gboolean                 details)
 {
 	gchar *fts;
 	gchar *query;
-	const gchar *show_all_str;
 	gboolean success;
 
-	show_all_str = show_all ? "" : "?email tracker:available true .";
 	fts = get_fts_string (search_terms, use_or_operator);
 
 	if (fts) {
@@ -370,63 +375,42 @@ get_emails (TrackerClient *client,
 		                         "WHERE { "
 		                         "  ?email a nmo:Email ;"
 		                         "  fts:match \"%s\" ."
-		                         "  %s"
 		                         "} "
 		                         "ORDER BY ASC(nmo:messageSubject(?email)) ASC(nmo:receivedDate(?email))"
 		                         "OFFSET %d "
 		                         "LIMIT %d",
 		                         fts,
-		                         show_all_str,
 		                         search_offset,
 		                         search_limit);
 	} else {
 		query = g_strdup_printf ("SELECT nmo:receivedDate(?email) nmo:messageSubject(?email) nie:url(?email) "
 		                         "WHERE { "
 		                         "  ?email a nmo:Email ."
-		                         "  %s"
 		                         "} "
 		                         "ORDER BY ASC(nmo:messageSubject(?email)) ASC(nmo:receivedDate(?email))"
 		                         "OFFSET %d "
 		                         "LIMIT %d",
-		                         show_all_str,
 		                         search_offset,
 		                         search_limit);
 	}
 
-	success = get_emails_results (client, query, search_limit, details);
+	success = get_emails_results (connection, query, search_limit, details);
 	g_free (query);
 	g_free (fts);
 
 	return success;
 }
 
-static void
-get_files_foreach (gpointer value,
-                   gpointer user_data)
-{
-	gchar **data;
-	gboolean details;
-
-	data = value;
-	details = GPOINTER_TO_INT (user_data);
-
-	if (details && data[1] && *data[1]) {
-		g_print ("  %s (%s)\n", data[1], data[0]);
-	} else {
-		g_print ("  %s\n", data[1]);
-	}
-}
-
 static gboolean
-get_files_results (TrackerClient *client,
-                   const gchar   *query,
-                   gint           search_limit,
-		   gboolean       details)
+get_files_results (TrackerSparqlConnection *connection,
+                   const gchar             *query,
+                   gint                     search_limit,
+                   gboolean                 details)
 {
 	GError *error = NULL;
-	GPtrArray *results;
+	TrackerSparqlCursor *cursor;
 
-	results = tracker_resources_sparql_query (client, query, &error);
+	cursor = tracker_sparql_connection_query (connection, query, NULL, &error);
 
 	if (error) {
 		g_printerr ("%s, %s\n",
@@ -437,40 +421,47 @@ get_files_results (TrackerClient *client,
 		return FALSE;
 	}
 
-	if (!results) {
+	if (!cursor) {
 		g_print ("%s\n",
 		         _("No files were found"));
 	} else {
-		g_print (g_dngettext (NULL,
-		                      "File: %d",
-		                      "Files: %d",
-		                      results->len),
-		         results->len);
+		gint count = 0;
+
+		g_print ("%s:\n", _("Files"));
+
+		while (tracker_sparql_cursor_next (cursor, NULL, NULL)) {
+			if (details) {
+				g_print ("  %s (%s)\n",
+				         tracker_sparql_cursor_get_string (cursor, 1, NULL),
+				         tracker_sparql_cursor_get_string (cursor, 0, NULL));
+			} else {
+				g_print ("  %s\n",
+				         tracker_sparql_cursor_get_string (cursor, 1, NULL));
+			}
+
+			count++;
+		}
+
 		g_print ("\n");
 
-		g_ptr_array_foreach (results,
-		                     get_files_foreach,
-		                     GINT_TO_POINTER (details));
-
-		if (results->len >= search_limit) {
+		if (count >= search_limit) {
 			show_limit_warning ();
 		}
 
-		g_ptr_array_foreach (results, (GFunc) g_strfreev, NULL);
-		g_ptr_array_free (results, TRUE);
+		g_object_unref (cursor);
 	}
 
 	return TRUE;
 }
 
 static gboolean
-get_document_files (TrackerClient *client,
-                    GStrv          search_terms,
-                    gboolean       show_all,
-                    gint           search_offset,
-                    gint           search_limit,
-                    gboolean       use_or_operator,
-		    gboolean       details)
+get_document_files (TrackerSparqlConnection *connection,
+                    GStrv                    search_terms,
+                    gboolean                 show_all,
+                    gint                     search_offset,
+                    gint                     search_limit,
+                    gboolean                 use_or_operator,
+                    gboolean                 details)
 {
 	gchar *fts;
 	gchar *query;
@@ -508,7 +499,7 @@ get_document_files (TrackerClient *client,
 		                         search_limit);
 	}
 
-	success = get_files_results (client, query, search_limit, details);
+	success = get_files_results (connection, query, search_limit, details);
 	g_free (query);
 	g_free (fts);
 
@@ -516,13 +507,13 @@ get_document_files (TrackerClient *client,
 }
 
 static gboolean
-get_video_files (TrackerClient *client,
-                 GStrv          search_terms,
-                 gboolean       show_all,
-                 gint           search_offset,
-                 gint           search_limit,
-                 gboolean       use_or_operator,
-		 gboolean       details)
+get_video_files (TrackerSparqlConnection *connection,
+                 GStrv                    search_terms,
+                 gboolean                 show_all,
+                 gint                     search_offset,
+                 gint                     search_limit,
+                 gboolean                 use_or_operator,
+                 gboolean                 details)
 {
 	gchar *fts;
 	gchar *query;
@@ -560,7 +551,7 @@ get_video_files (TrackerClient *client,
 		                         search_limit);
 	}
 
-	success = get_files_results (client, query, search_limit, details);
+	success = get_files_results (connection, query, search_limit, details);
 	g_free (query);
 	g_free (fts);
 
@@ -568,13 +559,13 @@ get_video_files (TrackerClient *client,
 }
 
 static gboolean
-get_image_files (TrackerClient *client,
-                 GStrv          search_terms,
-                 gboolean       show_all,
-                 gint           search_offset,
-                 gint           search_limit,
-		 gboolean       use_or_operator,
-		 gboolean       details)
+get_image_files (TrackerSparqlConnection *connection,
+                 GStrv                    search_terms,
+                 gboolean                 show_all,
+                 gint                     search_offset,
+                 gint                     search_limit,
+                 gboolean                 use_or_operator,
+                 gboolean                 details)
 {
 	gchar *fts;
 	gchar *query;
@@ -612,7 +603,7 @@ get_image_files (TrackerClient *client,
 		                         search_limit);
 	}
 
-	success = get_files_results (client, query, search_limit, details);
+	success = get_files_results (connection, query, search_limit, details);
 	g_free (query);
 	g_free (fts);
 
@@ -620,13 +611,13 @@ get_image_files (TrackerClient *client,
 }
 
 static gboolean
-get_music_files (TrackerClient *client,
-                 GStrv          search_terms,
-                 gboolean       show_all,
-                 gint           search_offset,
-                 gint           search_limit,
-                 gboolean       use_or_operator,
-		 gboolean       details)
+get_music_files (TrackerSparqlConnection *connection,
+                 GStrv                    search_terms,
+                 gboolean                 show_all,
+                 gint                     search_offset,
+                 gint                     search_limit,
+                 gboolean                 use_or_operator,
+                 gboolean                 details)
 {
 	gchar *fts;
 	gchar *query;
@@ -664,31 +655,23 @@ get_music_files (TrackerClient *client,
 		                         search_limit);
 	}
 
-	success = get_files_results (client, query, search_limit, details);
+	success = get_files_results (connection, query, search_limit, details);
 	g_free (query);
 	g_free (fts);
 
 	return success;
 }
 
-static void
-get_music_foreach (gpointer value,
-                   gpointer user_data)
-{
-	gchar **data = value;
-
-	g_print ("  '%s' (%s)\n", data[1], data[0]);
-}
-
 static gboolean
-get_music_artists (TrackerClient *client,
-                   GStrv          search_terms,
-                   gint           search_offset,
-                   gint           search_limit,
-                   gboolean       use_or_operator)
+get_music_artists (TrackerSparqlConnection *connection,
+                   GStrv                    search_terms,
+                   gint                     search_offset,
+                   gint                     search_limit,
+                   gboolean                 use_or_operator,
+                   gboolean                 details)
 {
 	GError *error = NULL;
-	GPtrArray *results;
+	TrackerSparqlCursor *cursor;
 	gchar *fts;
 	gchar *query;
 
@@ -722,7 +705,7 @@ get_music_artists (TrackerClient *client,
 
 	g_free (fts);
 
-	results = tracker_resources_sparql_query (client, query, &error);
+	cursor = tracker_sparql_connection_query (connection, query, NULL, &error);
 	g_free (query);
 
 	if (error) {
@@ -734,41 +717,48 @@ get_music_artists (TrackerClient *client,
 		return FALSE;
 	}
 
-	if (!results) {
+	if (!cursor) {
 		g_print ("%s\n",
 		         _("No artists were found"));
 	} else {
-		g_print (g_dngettext (NULL,
-		                      "Artist: %d",
-		                      "Artists: %d",
-		                      results->len),
-		         results->len);
+		gint count = 0;
+
+		g_print ("%s:\n", _("Artists"));
+
+		while (tracker_sparql_cursor_next (cursor, NULL, NULL)) {
+			if (details) {
+				g_print ("  '%s' (%s)\n",
+				         tracker_sparql_cursor_get_string (cursor, 1, NULL),
+				         tracker_sparql_cursor_get_string (cursor, 0, NULL));
+			} else {
+				g_print ("  '%s'\n",
+				         tracker_sparql_cursor_get_string (cursor, 1, NULL));
+			}
+			count++;
+		}
+
 		g_print ("\n");
 
-		g_ptr_array_foreach (results,
-		                     get_music_foreach,
-		                     NULL);
-
-		if (results->len >= search_limit) {
+		if (count >= search_limit) {
 			show_limit_warning ();
 		}
 
-		g_ptr_array_foreach (results, (GFunc) g_strfreev, NULL);
-		g_ptr_array_free (results, TRUE);
+		g_object_unref (cursor);
 	}
 
 	return TRUE;
 }
 
 static gboolean
-get_music_albums (TrackerClient *client,
-                  GStrv          search_words,
-                  gint           search_offset,
-                  gint           search_limit,
-                  gboolean       use_or_operator)
+get_music_albums (TrackerSparqlConnection *connection,
+                  GStrv                    search_words,
+                  gint                     search_offset,
+                  gint                     search_limit,
+                  gboolean                 use_or_operator,
+                  gboolean                 details)
 {
 	GError *error = NULL;
-	GPtrArray *results;
+	TrackerSparqlCursor *cursor;
 	gchar *fts;
 	gchar *query;
 
@@ -800,7 +790,7 @@ get_music_albums (TrackerClient *client,
 
 	g_free (fts);
 
-	results = tracker_resources_sparql_query (client, query, &error);
+	cursor = tracker_sparql_connection_query (connection, query, NULL, &error);
 	g_free (query);
 
 	if (error) {
@@ -812,40 +802,126 @@ get_music_albums (TrackerClient *client,
 		return FALSE;
 	}
 
-	if (!results) {
+	if (!cursor) {
 		g_print ("%s\n",
 		         _("No music was found"));
 	} else {
-		g_print (g_dngettext (NULL,
-		                      "Album: %d",
-		                      "Albums: %d",
-		                      results->len),
-		         results->len);
+		gint count = 0;
+
+		g_print ("%s:\n", _("Albums"));
+
+		while (tracker_sparql_cursor_next (cursor, NULL, NULL)) {
+			if (details) {
+				g_print ("  '%s' (%s)\n",
+				         tracker_sparql_cursor_get_string (cursor, 1, NULL),
+				         tracker_sparql_cursor_get_string (cursor, 0, NULL));
+			} else {
+				g_print ("  '%s'\n",
+				         tracker_sparql_cursor_get_string (cursor, 1, NULL));
+			}
+			count++;
+		}
+
 		g_print ("\n");
 
-		g_ptr_array_foreach (results,
-		                     get_music_foreach,
-		                     NULL);
-
-		if (results->len >= search_limit) {
+		if (count >= search_limit) {
 			show_limit_warning ();
 		}
 
-		g_ptr_array_foreach (results, (GFunc) g_strfreev, NULL);
-		g_ptr_array_free (results, TRUE);
+		g_object_unref (cursor);
 	}
 
 	return TRUE;
 }
 
 static gboolean
-get_files (TrackerClient *client,
-           GStrv          search_terms,
-           gboolean       show_all,
-           gint           search_offset,
-           gint           search_limit,
-	   gboolean       use_or_operator,
-	   gboolean       details)
+get_feeds (TrackerSparqlConnection *connection,
+           GStrv                    search_terms,
+           gint                     search_offset,
+           gint                     search_limit,
+           gboolean                 use_or_operator)
+{
+	GError *error = NULL;
+	TrackerSparqlCursor *cursor;
+	gchar *fts;
+	gchar *query;
+
+	fts = get_fts_string (search_terms, use_or_operator);
+
+	if (fts) {
+		query = g_strdup_printf ("SELECT ?feed nie:title(?feed) "
+		                         "WHERE {"
+		                         "  ?feed a mfo:FeedMessage ;"
+		                         "  fts:match \"%s\" . "
+		                         "} "
+		                         "ORDER BY ASC(nie:title(?feed)) "
+		                         "OFFSET %d "
+		                         "LIMIT %d",
+		                         fts,
+		                         search_offset,
+		                         search_limit);
+	} else {
+		query = g_strdup_printf ("SELECT ?feed nie:title(?feed) "
+		                         "WHERE {"
+		                         "  ?feed a mfo:FeedMessage ."
+		                         "} "
+		                         "ORDER BY ASC(nie:title(?feed)) "
+		                         "OFFSET %d "
+		                         "LIMIT %d",
+		                         search_offset,
+		                         search_limit);
+	}
+
+	g_free (fts);
+
+	cursor = tracker_sparql_connection_query (connection, query, NULL, &error);
+	g_free (query);
+
+	if (error) {
+		g_printerr ("%s, %s\n",
+		            _("Could not get search results"),
+		            error->message);
+		g_error_free (error);
+
+		return FALSE;
+	}
+
+	if (!cursor) {
+		g_print ("%s\n",
+		         _("No feeds were found"));
+	} else {
+		gint count = 0;
+
+		g_print ("%s:\n", _("Feeds"));
+
+		while (tracker_sparql_cursor_next (cursor, NULL, NULL)) {
+			g_print ("  %s (%s)\n",
+			         tracker_sparql_cursor_get_string (cursor, 0, NULL),
+			         tracker_sparql_cursor_get_string (cursor, 1, NULL));
+
+			count++;
+		}
+
+		g_print ("\n");
+
+		if (count >= search_limit) {
+			show_limit_warning ();
+		}
+
+		g_object_unref (cursor);
+	}
+
+	return TRUE;
+}
+
+static gboolean
+get_files (TrackerSparqlConnection *connection,
+           GStrv                    search_terms,
+           gboolean                 show_all,
+           gint                     search_offset,
+           gint                     search_limit,
+           gboolean                 use_or_operator,
+           gboolean                 details)
 {
 	gchar *fts;
 	gchar *query;
@@ -883,7 +959,7 @@ get_files (TrackerClient *client,
 		                         search_limit);
 	}
 
-	success = get_files_results (client, query, search_limit, details);
+	success = get_files_results (connection, query, search_limit, details);
 	g_free (query);
 	g_free (fts);
 
@@ -891,13 +967,13 @@ get_files (TrackerClient *client,
 }
 
 static gboolean
-get_folders (TrackerClient *client,
-             GStrv          search_terms,
-             gboolean       show_all,
-             gint           search_offset,
-             gint           search_limit,
-	     gboolean       use_or_operator,
-	     gboolean       details)
+get_folders (TrackerSparqlConnection *connection,
+             GStrv                    search_terms,
+             gboolean                 show_all,
+             gint                     search_offset,
+             gint                     search_limit,
+             gboolean                 use_or_operator,
+             gboolean                 details)
 {
 	gchar *fts;
 	gchar *query;
@@ -935,78 +1011,24 @@ get_folders (TrackerClient *client,
 		                         search_limit);
 	}
 
-	success = get_files_results (client, query, search_limit, details);
+	success = get_files_results (connection, query, search_limit, details);
 	g_free (query);
 	g_free (fts);
 
 	return success;
 }
 
-static void
-get_all_by_search_foreach (gpointer value,
-                           gpointer user_data)
-{
-	gchar **metadata;
-	gboolean details;
-
-	metadata = value;
-	details = GPOINTER_TO_INT (user_data);
-
-	if (!metadata || !*metadata) {
-		return;
-	}
-
-	if (G_LIKELY (!details)) {
-		gchar **p;
-		gint i;
-
-		for (p = metadata, i = 0; *p; p++, i++) {
-			if (i == 0) {
-				g_print ("  %s", *p);
-			}
-		}
-	} else {
-		const gchar *urn;
-		const gchar *mime_type;
-		const gchar *class;
-
-		urn = metadata[0];
-		mime_type = metadata[1];
-		class = metadata[2];
-
-		if (mime_type && mime_type[0] == '\0') {
-			mime_type = NULL;
-		}
-		
-		if (mime_type) {
-			g_print ("  %s\n"
-			         "    %s\n"
-			         "    %s\n", 
-			         urn, 
-			         mime_type, 
-			         class);
-		} else {
-			g_print ("  %s\n"
-			         "    %s\n", 
-			         urn, 
-			         class);
-		}
-	}
-
-	g_print ("\n");
-}
-
 static gboolean
-get_all_by_search (TrackerClient *client,
-                   GStrv          search_words,
-                   gboolean       show_all,
-                   gint           search_offset,
-                   gint           search_limit,
-                   gboolean       use_or_operator,
-                   gboolean       details)
+get_all_by_search (TrackerSparqlConnection *connection,
+                   GStrv                    search_words,
+                   gboolean                 show_all,
+                   gint                     search_offset,
+                   gint                     search_limit,
+                   gboolean                 use_or_operator,
+                   gboolean                 details)
 {
 	GError *error = NULL;
-	GPtrArray *results;
+	TrackerSparqlCursor *cursor;
 	gchar *fts;
 	gchar *query;
 	const gchar *show_all_str;
@@ -1048,7 +1070,7 @@ get_all_by_search (TrackerClient *client,
 
 	g_free (fts);
 
-	results = tracker_resources_sparql_query (client, query, &error);
+	cursor = tracker_sparql_connection_query (connection, query, NULL, &error);
 	g_free (query);
 
 	if (error) {
@@ -1060,27 +1082,58 @@ get_all_by_search (TrackerClient *client,
 		return FALSE;
 	}
 
-	if (!results) {
+	if (!cursor) {
 		g_print ("%s\n",
 		         _("No results were found matching your query"));
 	} else {
-		g_print (g_dngettext (NULL,
-		                      "Result: %d",
-		                      "Results: %d",
-		                      results->len),
-		         results->len);
+		gint count = 0;
+
+		g_print ("%s:\n", _("Results"));
+
+		while (tracker_sparql_cursor_next (cursor, NULL, NULL)) {
+			if (details) {
+				const gchar *urn;
+				const gchar *mime_type;
+				const gchar *class;
+
+				g_print ("cols:%d\n", tracker_sparql_cursor_get_n_columns (cursor));
+
+				urn = tracker_sparql_cursor_get_string (cursor, 0, NULL);
+				mime_type = tracker_sparql_cursor_get_string (cursor, 1, NULL);
+				class = tracker_sparql_cursor_get_string (cursor, 2, NULL);
+
+				if (mime_type && mime_type[0] == '\0') {
+					mime_type = NULL;
+				}
+
+				if (mime_type) {
+					g_print ("  %s\n"
+					         "    %s\n"
+					         "    %s\n",
+					         urn,
+					         mime_type,
+					         class);
+				} else {
+					g_print ("  %s\n"
+					         "    %s\n",
+					         urn,
+					         class);
+				}
+			} else {
+				g_print ("  %s\n",
+				         tracker_sparql_cursor_get_string (cursor, 0, NULL));
+			}
+
+			count++;
+		}
+
 		g_print ("\n");
 
-		g_ptr_array_foreach (results,
-		                     get_all_by_search_foreach,
-		                     GINT_TO_POINTER (details));
-
-		if (results->len >= search_limit) {
+		if (count >= search_limit) {
 			show_limit_warning ();
 		}
 
-		g_ptr_array_foreach (results, (GFunc) g_strfreev, NULL);
-		g_ptr_array_free (results, TRUE);
+		g_object_unref (cursor);
 	}
 
 	return TRUE;
@@ -1089,8 +1142,9 @@ get_all_by_search (TrackerClient *client,
 int
 main (int argc, char **argv)
 {
-	TrackerClient *client;
+	TrackerSparqlConnection *connection;
 	GOptionContext *context;
+	GError *error = NULL;
 	gchar *summary;
 
 	setlocale (LC_ALL, "");
@@ -1128,6 +1182,7 @@ main (int argc, char **argv)
 	}
 
 	if (!music_albums && !music_artists && !music_files &&
+	    !feeds &&
 	    !image_files &&
 	    !video_files &&
 	    !document_files &&
@@ -1154,54 +1209,66 @@ main (int argc, char **argv)
 		g_thread_init (NULL);
 	}
 
-	/* Check terms are not stopwords */
+
+#if HAVE_TRACKER_FTS
+	/* Only check stopwords if FTS is enabled */
 	if (terms) {
 		TrackerLanguage *language;
-		GHashTable *stop_words;
-		const gchar *stop_word_found;
+		gboolean stop_words_found;
 		gchar **p;
 
-		language = tracker_language_new (NULL);
-		stop_words = tracker_language_get_stop_words (language);
+		/* Check terms don't have additional quotes */
+		for (p = terms; *p; p++) {
+			gchar *term = *p;
+			gint end = strlen (term) - 1;
 
-		for (p = terms, stop_word_found = NULL; *p && !stop_word_found; p++) {
-			gpointer data;
-			gchar *up = g_utf8_strdown (*p, -1);
-
-			data = g_hash_table_lookup (stop_words, up);
-			if (data) {
-				stop_word_found = *p;
+			if ((term[0] == '"' && term[end] == '"') ||
+			    (term[0] == '\'' && term[end] == '\'')) {
+				/* We never have a quote JUST at the end */
+				term[0] = term[end] = ' ';
+				g_strstrip (term);
 			}
-			g_free (up);
 		}
 
-		if (stop_word_found) {
-			g_printerr (_("Search term '%s' is a stop word."),
-			            stop_word_found);
-			g_printerr ("\n\n");
-			g_printerr (_("Stop words are common words which are "
-			              "ignored during the indexing process."));
-			g_printerr ("\n");
-			g_printerr (_("This means this search term will never "
-			              "be found when matching FTS entries."));
-			g_printerr ("\n\n");
+		/* Check if terms are stopwords, and warn if so */
+		language = tracker_language_new (NULL);
+		stop_words_found = FALSE;
+		for (p = terms; *p; p++) {
+			gchar *down;
 
-			g_option_context_free (context);
-			g_object_unref (language);
+			down = g_utf8_strdown (*p, -1);
 
-			return EXIT_FAILURE;
+			if (tracker_language_is_stop_word (language, down)) {
+				g_printerr (_("Search term '%s' is a stop word."),
+				            down);
+				g_printerr ("\n");
+
+				stop_words_found = TRUE;
+			}
+
+			g_free (down);
+		}
+
+		if (stop_words_found) {
+			g_printerr (_("Stop words are common words which "
+			              "may be ignored during the indexing "
+			              "process."));
+			g_printerr ("\n\n");
 		}
 
 		g_object_unref (language);
 	}
+#endif
 
 	g_option_context_free (context);
 
-	client = tracker_client_new (0, G_MAXINT);
+	connection = tracker_sparql_connection_get (&error);
 
-	if (!client) {
-		g_printerr ("%s\n",
-		            _("Could not establish a DBus connection to Tracker"));
+	if (!connection) {
+		g_printerr ("%s: %s\n",
+		            _("Could not establish a connection to Tracker"),
+		            error ? error->message : _("No error given"));
+		g_clear_error (&error);
 		return EXIT_FAILURE;
 	}
 
@@ -1212,8 +1279,8 @@ main (int argc, char **argv)
 	if (files) {
 		gboolean success;
 
-		success = get_files (client, terms, all, offset, limit, or_operator, detailed);
-		g_object_unref (client);
+		success = get_files (connection, terms, all, offset, limit, or_operator, detailed);
+		g_object_unref (connection);
 
 		return success ? EXIT_SUCCESS : EXIT_FAILURE;
 	}
@@ -1221,8 +1288,8 @@ main (int argc, char **argv)
 	if (folders) {
 		gboolean success;
 
-		success = get_folders (client, terms, all, offset, limit, or_operator, detailed);
-		g_object_unref (client);
+		success = get_folders (connection, terms, all, offset, limit, or_operator, detailed);
+		g_object_unref (connection);
 
 		return success ? EXIT_SUCCESS : EXIT_FAILURE;
 	}
@@ -1230,8 +1297,8 @@ main (int argc, char **argv)
 	if (music_albums) {
 		gboolean success;
 
-		success = get_music_albums (client, terms, offset, limit, or_operator);
-		g_object_unref (client);
+		success = get_music_albums (connection, terms, offset, limit, or_operator, detailed);
+		g_object_unref (connection);
 
 		return success ? EXIT_SUCCESS : EXIT_FAILURE;
 	}
@@ -1239,8 +1306,8 @@ main (int argc, char **argv)
 	if (music_artists) {
 		gboolean success;
 
-		success = get_music_artists (client, terms, offset, limit, or_operator);
-		g_object_unref (client);
+		success = get_music_artists (connection, terms, offset, limit, or_operator, detailed);
+		g_object_unref (connection);
 
 		return success ? EXIT_SUCCESS : EXIT_FAILURE;
 	}
@@ -1248,8 +1315,17 @@ main (int argc, char **argv)
 	if (music_files) {
 		gboolean success;
 
-		success = get_music_files (client, terms, all, offset, limit, or_operator, detailed);
-		g_object_unref (client);
+		success = get_music_files (connection, terms, all, offset, limit, or_operator, detailed);
+		g_object_unref (connection);
+
+		return success ? EXIT_SUCCESS : EXIT_FAILURE;
+	}
+
+	if (feeds) {
+		gboolean success;
+
+		success = get_feeds (connection, terms, offset, limit, or_operator);
+		g_object_unref (connection);
 
 		return success ? EXIT_SUCCESS : EXIT_FAILURE;
 	}
@@ -1257,8 +1333,8 @@ main (int argc, char **argv)
 	if (image_files) {
 		gboolean success;
 
-		success = get_image_files (client, terms, all, offset, limit, or_operator, detailed);
-		g_object_unref (client);
+		success = get_image_files (connection, terms, all, offset, limit, or_operator, detailed);
+		g_object_unref (connection);
 
 		return success ? EXIT_SUCCESS : EXIT_FAILURE;
 	}
@@ -1266,8 +1342,8 @@ main (int argc, char **argv)
 	if (video_files) {
 		gboolean success;
 
-		success = get_video_files (client, terms, all, offset, limit, or_operator, detailed);
-		g_object_unref (client);
+		success = get_video_files (connection, terms, all, offset, limit, or_operator, detailed);
+		g_object_unref (connection);
 
 		return success ? EXIT_SUCCESS : EXIT_FAILURE;
 	}
@@ -1275,8 +1351,8 @@ main (int argc, char **argv)
 	if (document_files) {
 		gboolean success;
 
-		success = get_document_files (client, terms, all, offset, limit, or_operator, detailed);
-		g_object_unref (client);
+		success = get_document_files (connection, terms, all, offset, limit, or_operator, detailed);
+		g_object_unref (connection);
 
 		return success ? EXIT_SUCCESS : EXIT_FAILURE;
 	}
@@ -1284,8 +1360,8 @@ main (int argc, char **argv)
 	if (emails) {
 		gboolean success;
 
-		success = get_emails (client, terms, all, offset, limit, or_operator, detailed);
-		g_object_unref (client);
+		success = get_emails (connection, terms, all, offset, limit, or_operator, detailed);
+		g_object_unref (connection);
 
 		return success ? EXIT_SUCCESS : EXIT_FAILURE;
 	}
@@ -1293,8 +1369,8 @@ main (int argc, char **argv)
 	if (contacts) {
 		gboolean success;
 
-		success = get_contacts (client, terms, all, offset, limit, or_operator, detailed);
-		g_object_unref (client);
+		success = get_contacts (connection, terms, all, offset, limit, or_operator, detailed);
+		g_object_unref (connection);
 
 		return success ? EXIT_SUCCESS : EXIT_FAILURE;
 	}
@@ -1302,13 +1378,13 @@ main (int argc, char **argv)
 	if (terms) {
 		gboolean success;
 
-		success = get_all_by_search (client, terms, all, offset, limit, or_operator, detailed);
-		g_object_unref (client);
+		success = get_all_by_search (connection, terms, all, offset, limit, or_operator, detailed);
+		g_object_unref (connection);
 
 		return success ? EXIT_SUCCESS : EXIT_FAILURE;
 	}
 
-	g_object_unref (client);
+	g_object_unref (connection);
 
 	return EXIT_SUCCESS;
 }
